import numpy
import os

import NLEngine

from API import LabFloorImporterPlugin
from API import LabFloorItem
from NL.Analysis.ElectronDensity import ElectronDensity
from NL.Analysis.GridValues import GridValues
from NL.ComputerScienceUtilities.NLFlag import Spin
from NL.CommonConcepts.PhysicalQuantity import Angstrom

class NPZLabFloor(LabFloorImporterPlugin):
    """
    Class for handling the importing of NPZ-files as LabFloor items.
    """

    def scan(self, filename):
        """
        Scans a file to check if it is supported by the plugin

        @param filename : The path to be scanned.
        @type           : string

        @return A list of LabFloorItems
        """
        # Setup a vector for the LabFloorItems that will be returned.
        lab_floor_items = []

        # Determine extension
        basename = os.path.basename(filename)
        no_extension_name, extension = os.path.splitext(basename)

        # Return an empty list if the extension isn't ".npz"
        if extension != '.npz':
            return []

        item = LabFloorItem(ElectronDensity,
                            title='NPZ Electron Density',
                            tool_tip='NPZ Electron Density')

        # Add to the list of items.
        lab_floor_items.append(item)

        # Return the list of items.
        return lab_floor_items

    def load(self, filename):
        """
        Load the desired object in memory.

        @param filename : The path of the NPZ-file.
        @type           : string

        @return Desired object (MoleculeConfiguration)
        """

        # Read the file
        npz = numpy.load(filename)

        # Create an "empty" ElectronDensity object.
        electron_density = ElectronDensity.__new__(ElectronDensity)

        # We will now fill out a dictionary that contains the information
        # needed by the ElectronDensity class.
        data = {}

        # The "data" key is the electron density. The units must be given in the
        # "data_unit" key. The array should have have the x-axis as the first
        # dimension, the y-axis as the second, and the z-axis as the third.
        data['data'] = npz['density']

        # The data in "data" has no units so they are assigned here.
        data['data_unit'] = 1.0 * Angstrom**-3

        # Set the origin to be at zero.
        data['origo'] = numpy.zeros(3)

        # The cell must be given in Bohr units.
        data['cell'] = npz['cell']

        # The boundary conditions are expressed as a list of 6 numbers that should
        # map to:
        # { Dirichlet, Neumann, Periodic, Multipole };
        # A value of 2 corresponds to "Periodic".
        data['boundary_conditions'] = [2, 2, 2, 2, 2, 2]

        # Construct the GridValues specific part of the object.
        GridValues._populateFromDict(electron_density, data)

        # Set the spin_type to unpolarized.
        spin_type = NLEngine.UNPOLARIZED
        electron_density._AnalysisSpin__spin_type = spin_type

        sp = Spin.All
        electron_density._AnalysisSpin__spin = sp
        electron_density._setSupportedSpins(sp)

        return electron_density
